@@ -31,9 +31,6 @@
 #include "acme.h"
 #include "eggcellrendererkeys.h"
 
-#define BACKSPACE_KEYCODE 22
-#define C_KEYCODE 54
-
 enum {
 	PATH_COL,
 	DESC_COL,
@@ -41,6 +38,12 @@ enum {
 	NUM_COLS
 };
 
+typedef struct
+{
+	GtkTreeView *tree_view;
+	GtkTreePath *path;
+} IdleData;
+
 GConfClient *conf_client;
 
 #define SELECTION_NAME "_ACME_SELECTION"
@@ -104,19 +107,19 @@ keys_edited_callback (GtkCellRendererText *cell, const char *path_string,
 	if (gconf_path == NULL)
 		return;
 
-	if (keycode == C_KEYCODE)
+	if (keyval == GDK_C || keyval == GDK_c)
 	{
 		gtk_tree_path_free (path);
 		return;
 	}
 
-	if (keycode == BACKSPACE_KEYCODE)
+	if (keyval == GDK_BackSpace)
 	{
-		value = 0;
+		value = -1;
 		value_str = g_strdup (_("Disabled"));
 	} else {
 		value = keycode;
-		value_str = g_strdup_printf ("%d", value);
+		value_str = g_strdup (_("Enabled"));
 	}
 
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
@@ -128,6 +131,48 @@ keys_edited_callback (GtkCellRendererText *cell, const char *path_string,
 	gtk_tree_path_free (path);
 }
 
+static gboolean
+real_start_editing_cb (IdleData *idle_data)
+{
+	gtk_widget_grab_focus (GTK_WIDGET (idle_data->tree_view));
+	gtk_tree_view_set_cursor (idle_data->tree_view,
+			idle_data->path,
+			gtk_tree_view_get_column (idle_data->tree_view,
+				DESC_COL),
+			TRUE);
+
+	gtk_tree_path_free (idle_data->path);
+	g_free (idle_data);
+	return FALSE;
+}
+
+static gboolean
+start_editing_cb (GtkTreeView *tree_view, GdkEventButton *event,
+		gpointer user_dta)
+{
+	GtkTreePath *path;
+
+	if (event->window != gtk_tree_view_get_bin_window (tree_view))
+		return FALSE;
+
+	if (gtk_tree_view_get_path_at_pos (tree_view,
+				(gint) event->x,
+				(gint) event->y,
+				&path, NULL,
+				NULL, NULL))
+	{
+		IdleData *idle_data;
+
+		idle_data = g_new (IdleData, 1);
+		idle_data->tree_view = tree_view;
+		idle_data->path = path;
+		g_signal_stop_emission_by_name (G_OBJECT (tree_view),
+				"button_press_event");
+		g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
+	}
+	return TRUE;
+}
+
 static void
 init_content_helper (GtkListStore *store, const char *path,
 		const char *desc, int keyval)
@@ -136,7 +181,7 @@ init_content_helper (GtkListStore *store, const char *path,
 	char *keyval_str;
 
 	if (keyval > 0)
-		keyval_str = g_strdup_printf ("%d", keyval);
+		keyval_str = g_strdup (_("Enabled"));
 	else
 		keyval_str = g_strdup (_("Disabled"));
 
@@ -188,13 +233,16 @@ init_columns (GtkTreeView *treeview)
 			G_CALLBACK (keys_edited_callback),
 			gtk_tree_view_get_model (treeview));
 
-	column = gtk_tree_view_column_new_with_attributes (_("Keycode value"),
+	column = gtk_tree_view_column_new_with_attributes (_("Status"),
 			renderer,
 			"text", KEYVAL_COL,
 			NULL);
 	gtk_tree_view_column_set_fixed_width (column, 20);
 
 	gtk_tree_view_append_column (treeview, column);
+
+	g_signal_connect (G_OBJECT (treeview), "button_press_event",
+			G_CALLBACK (start_editing_cb), NULL);
 }
 
 static void
