@@ -1,4 +1,5 @@
 /* ACME
+ * init_tray (acme);
  * Copyright (C) 2001 Bastien Nocera <hadess@hadess.net>
  *
  * acme.c
@@ -33,6 +34,7 @@
 #include <gnome.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include "eggtrayicon.h"
 
 #include "acme.h"
 #include "g-volume.h"
@@ -54,7 +56,11 @@ typedef struct {
 	int NumLockMask, CapsLockMask, ScrollLockMask;
 	GConfClient *conf_client;
 	guint dialog_timeout;
-	int lock_fd;
+
+	/* Tray icon */
+	EggTrayIcon *tray_icon;
+	GtkTooltips *tray_icon_tooltip;
+	GtkWidget *popup_menu;
 } Acme;
 
 enum {
@@ -113,16 +119,6 @@ out:
 static void
 acme_exit (Acme *acme)
 {
-	if (acme->lock_fd > 0)
-	{
-		char *path;
-
-		close (acme->lock_fd);
-		path = g_strdup_printf ("%s/.acme.lock", g_get_home_dir());
-		unlink (path);
-		g_free (path);
-	}
-
 	exit (0);
 }
 
@@ -145,9 +141,17 @@ acme_error (char * msg)
 }
 
 static void
-execute (char * cmd)
+execute (char *cmd, gboolean sync)
 {
-	if (g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL) == FALSE)
+	gboolean retval;
+
+	if (sync == TRUE)
+		retval = g_spawn_command_line_sync
+			(cmd, NULL, NULL, NULL, NULL);
+	else
+		retval = g_spawn_command_line_async (cmd, NULL);
+
+	if (retval == FALSE)
 	{
 		char *msg;
 
@@ -161,6 +165,28 @@ execute (char * cmd)
 	}
 }
 
+static void
+execute_this_or_that (char *cmd1, char *cmd2)
+{
+	if (g_spawn_command_line_async (cmd1, NULL) == FALSE)
+	{
+		if (g_spawn_command_line_async (cmd2, NULL) == FALSE)
+		{
+			char *msg;
+
+			msg = g_strdup_printf
+				(_("Couldn't execute either command: %s\n"
+				   "or command: %s\n"
+				   "Verify that at least one of these commands"
+				   "exist."),
+				 cmd1, cmd2);
+
+			acme_error (msg);
+			g_free (msg);
+		}
+	}
+}
+
 static void
 acme_play_sound (Acme *acme)
 {
@@ -185,7 +211,7 @@ acme_play_sound (Acme *acme)
 		return;
 	}
 
-	execute (command);
+	execute (command, FALSE);
 	g_free (command);
 }
 
@@ -290,18 +316,50 @@ static void
 grab_key (int key_code)
 {
 	gdk_error_trap_push ();
-	XGrabKey (GDK_DISPLAY (), key_code, AnyModifier,
-		  GDK_ROOT_WINDOW (), True,
-		  GrabModeAsync, GrabModeAsync);
+
+	XGrabKey (GDK_DISPLAY (), key_code,
+			0,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod5Mask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask | LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod5Mask | LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask | Mod5Mask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+	XGrabKey (GDK_DISPLAY (), key_code,
+			Mod2Mask | Mod5Mask | LockMask,
+			GDK_ROOT_WINDOW (), True,
+			GrabModeAsync, GrabModeAsync);
+
 	gdk_flush ();
-	if (gdk_error_trap_pop ()) {
+	if (gdk_error_trap_pop ())
+	{
 		char *error;
 
 		error = g_strdup_printf
 			(_("It seems that another application already has"
 			   " access to the multimedia keys.\n"
 			   "Key %d couldn't be bound.\n"
-			   "Close this application and restart ACME."),
+			   "Is another daemon already running ?"),
 			 key_code);
 		acme_error (error);
 		g_free (error);
@@ -318,7 +376,8 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	g_return_if_fail (entry->key != NULL);
 
 	/* Find the key that was modified */
-	for (i = 0; i < HANDLED_KEYS; i++) {
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
 		if (strcmp (entry->key, keys[i].key_config) == 0)
 		{
 			int key_code;
@@ -327,7 +386,8 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 			key_code = gconf_client_get_int (acme->conf_client,
 					keys[i].key_config,
 					NULL);
-			if (key_code > 0) {
+			if (key_code > 0)
+			{
 				grab_key (key_code);
 				keys[i].key_code = key_code;
 			}
@@ -335,11 +395,12 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 	}
 }
 
-static void
+static gboolean
 init_kbd (Acme *acme)
 {
 	GtkWidget *widget = acme->invisible;
 	int i;
+	gboolean retval = FALSE;
 
 	gtk_widget_show (widget);
 
@@ -349,7 +410,8 @@ init_kbd (Acme *acme)
 	gdk_window_set_events (gdk_get_default_root_window (),
 			GDK_KEY_PRESS_MASK);
 
-	for (i = 0; i < HANDLED_KEYS; i++) {
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
 		int tmp;
 
 		tmp = gconf_client_get_int (acme->conf_client,
@@ -358,7 +420,7 @@ init_kbd (Acme *acme)
 
 		keys[i].key_code = tmp;
 
-		if (tmp != -1)
+		if (tmp > 0)
 		{
 #ifdef DEBUG
 			g_print ("grabbed key %d for gconf key %s\n",
@@ -366,15 +428,127 @@ init_kbd (Acme *acme)
 					keys[i].key_config);
 #endif
 			grab_key (keys [i].key_code);
+			retval = TRUE;
 		}
 	}
 
-	for (i = 0; i < HANDLED_KEYS; i++) {
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
 		gconf_client_notify_add (acme->conf_client,
 				keys[i].key_config,
 				update_kbd_cb,
 				acme, NULL, NULL);
 	}
+
+	return retval;
+}
+
+static void
+prefs_activated (GtkMenuItem *menuitem, gpointer user_data)
+{
+	execute ("acme-properties", FALSE);
+}
+
+static void
+about_activated (GtkMenuItem *menuitem, gpointer user_data)
+{
+	static GtkWidget *about = NULL;
+	GdkPixbuf *pixbuf = NULL;
+	const gchar *authors[] = { "Bastien Nocera <hadess@hadess.net>", NULL };
+	const gchar *documenters[] = { NULL };
+	const gchar *translator_credits = _("translator_credits");
+
+	if (about != NULL)
+	{
+		gdk_window_raise (about->window);
+		gdk_window_show (about->window);
+		return;
+	}
+
+	pixbuf = gdk_pixbuf_new_from_file (ACME_DATA "keymap-48.png", NULL);
+
+	about = gnome_about_new(_("Acme"), VERSION,
+			_("(C) 2001, 2002 Bastien Nocera"),
+			_("Multimedia keys daemon"),
+			(const char **)authors,
+			(const char **)documenters,
+			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			pixbuf);
+
+	g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK
+			(gtk_widget_destroyed), &about);
+	g_object_add_weak_pointer (G_OBJECT (about),
+			(void**)&(about));
+
+	gtk_widget_show(about);
+}
+
+static gboolean
+tray_icon_release (GtkWidget *widget, GdkEventButton *event, Acme *acme)
+{
+	if (event->button == 3)
+	{
+		gtk_menu_popdown (GTK_MENU (acme->popup_menu));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+
+static gboolean
+tray_icon_press (GtkWidget *widget, GdkEventButton *event, Acme *acme)
+{
+	if (event->button == 3)
+	{
+		gtk_menu_popup (GTK_MENU (acme->popup_menu), NULL, NULL, NULL,
+				NULL, event->button, event->time);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void
+init_tray (Acme *acme)
+{
+	GtkWidget *image, *evbox, *item;
+
+	acme->tray_icon = egg_tray_icon_new ("Multimedia Keys daemon");
+	image = gtk_image_new_from_file (ACME_DATA "keymap-16.png");
+
+	acme->tray_icon_tooltip = gtk_tooltips_new ();
+	gtk_tooltips_set_tip (acme->tray_icon_tooltip,
+			GTK_WIDGET (acme->tray_icon),
+			_("Multimedia Keys daemon active"),
+			NULL);
+
+	/* Event box */
+	evbox = gtk_event_box_new ();
+	g_signal_connect (G_OBJECT (evbox), "button_press_event",
+			G_CALLBACK (tray_icon_press), (gpointer) acme);
+	g_signal_connect (G_OBJECT (evbox), "button_release_event",
+			G_CALLBACK (tray_icon_release), (gpointer) acme);
+
+	/* Popup menu */
+	acme->popup_menu = gtk_menu_new ();
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES,
+			NULL);
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (prefs_activated), (gpointer) acme);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (acme->popup_menu), item);
+
+	item = gtk_image_menu_item_new_from_stock (GNOME_STOCK_ABOUT,
+			NULL);
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (about_activated), (gpointer) acme);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (acme->popup_menu), item);
+
+	gtk_container_add (GTK_CONTAINER (evbox), image);
+	gtk_container_add (GTK_CONTAINER (acme->tray_icon), evbox);
+	gtk_widget_show_all (GTK_WIDGET (acme->tray_icon));
 }
 
 static void
@@ -475,9 +649,9 @@ do_eject_action (Acme *acme)
 	command = gconf_client_get_string (acme->conf_client,
 			"/apps/acme/eject_command", NULL);
 	if ((command != NULL) && (strcmp (command, "") != 0))
-		execute (command);
+		execute (command, TRUE);
 	else
-		execute ("eject");
+		execute ("eject", TRUE);
 
 	gtk_widget_set_sensitive (progress, TRUE);
 }
@@ -537,14 +711,16 @@ do_sound_action (Acme *acme, int type)
 		g_volume_mute_toggle(acme->volobj);
 		break;
 	case VOLUME_DOWN_KEY:
-		if (muted) {
+		if (muted)
+		{
 			g_volume_mute_toggle(acme->volobj);
 		} else {
 			g_volume_set_volume (acme->volobj, vol - VOLUME_STEP);
 		}
 		break;
 	case VOLUME_UP_KEY:
-		if (muted) {
+		if (muted)
+		{
 			g_volume_mute_toggle(acme->volobj);
 		} else {
 			g_volume_set_volume (acme->volobj, vol + VOLUME_STEP);
@@ -577,7 +753,8 @@ do_action (int type, Acme *acme)
 	case MUTE_KEY:
 	case VOLUME_DOWN_KEY:
 	case VOLUME_UP_KEY:
-		do_sound_action (acme, type);
+		if (acme->volobj != NULL)
+			do_sound_action (acme, type);
 		break;
 	case POWER_KEY:
 		do_exit_action (acme);
@@ -585,12 +762,32 @@ do_action (int type, Acme *acme)
 	case EJECT_KEY:
 		/* Do nothing, handled differently */
 		break;
+	case HOME_KEY:
+		execute ("nautilus", FALSE);
+		break;
+	case SEARCH_KEY:
+		execute ("gnome-search-tool", FALSE);
+		break;
+	case EMAIL_KEY:
+		execute_this_or_that ("evolution", "balsa");
+		break;
+	case SLEEP_KEY:
+		execute ("apm", TRUE);
+		break;
+	case SCREENSAVER_KEY:
+		execute ("xscreensaver-command -lock", FALSE);
+		break;
+	case FINANCE_KEY:
+		execute ("gnucash", FALSE);
+		break;
+	case HELP_KEY:
+		execute ("yelp", FALSE);
+		break;
 #ifdef USE_FBLEVEL
 	case BRIGHT_DOWN_KEY:
 	case BRIGHT_UP_KEY:
-		if (acme->levobj == NULL)
-			break;
-		do_brightness_action (acme, type);
+		if (acme->levobj != NULL)
+			do_brightness_action (acme, type);
 		break;
 #endif
 	default:
@@ -672,7 +869,7 @@ main (int argc, char *argv[])
 
 	if (acme_get_lock (acme) == FALSE)
 	{
-		g_print ("Acme is already running, exiting...\n");
+		g_print ("Daemon already running, exiting...\n");
 		acme_exit (acme);
 	}
 
@@ -681,7 +878,8 @@ main (int argc, char *argv[])
 
 	if (acme->xml == NULL) {
 		acme_error (_("Couldn't load the Glade file.\n"
-				"Make sure that ACME is properly installed."));
+				"Make sure that this daemon is properly"
+				" installed."));
 		exit (1);
 	}
 
@@ -695,31 +893,32 @@ main (int argc, char *argv[])
 			NULL);
 
 	acme->invisible = gtk_invisible_new ();
-	init_kbd (acme);
+	if (init_kbd (acme) == FALSE)
+	{
+		execute ("acme-properties", FALSE);
+		exit (0);
+	}
 	init_sm (acme);
-	gtk_widget_realize(acme->dialog);
+	init_tray (acme);
+	gtk_widget_realize (acme->dialog);
 	acme->dialog_timeout = 0;
 
 	/* initialise Volume handler */
 	acme->volobj = g_volume_new();
-	if (acme->volobj == NULL)
+	if (acme->volobj != NULL)
 	{
-		acme_error (_("Your system has no sound support, or ACME was"
-				" compiled without any sound support\n"
-				"ACME will now exit"));
-		exit (1);
-	}
-	g_signal_connect(G_OBJECT(acme->volobj),
-			"fd_problem",
-			(GCallback) vol_problem_cb, NULL);
+		g_signal_connect(G_OBJECT(acme->volobj),
+				"fd_problem",
+				G_CALLBACK (vol_problem_cb), NULL);
 
-	g_volume_set_use_pcm (acme->volobj,
-			gconf_client_get_bool (acme->conf_client,
-				"/apps/acme/use_pcm", NULL));
-	gconf_client_notify_add (acme->conf_client,
-			"/apps/acme/use_pcm",
-			update_use_pcm_cb,
-			acme, NULL, NULL);
+		g_volume_set_use_pcm (acme->volobj,
+				gconf_client_get_bool (acme->conf_client,
+					"/apps/acme/use_pcm", NULL));
+		gconf_client_notify_add (acme->conf_client,
+				"/apps/acme/use_pcm",
+				update_use_pcm_cb,
+				acme, NULL, NULL);
+	}
 
 #ifdef USE_FBLEVEL
 	/* initialise Frame Buffer level handler */
@@ -730,7 +929,7 @@ main (int argc, char *argv[])
 
 	g_signal_connect (GTK_OBJECT (acme->invisible),
 			    "event",
-			    (GCallback) event_cb, acme);
+			    G_CALLBACK (event_cb), acme);
 
 	gtk_main ();
 
